/********************************************************************/
/*                                                                  */
/*   Program Name :  monit                                          */
/*                                                                  */
/*      Version:  0.00  2002-05-26                                  */
/*                2.01  2002-10-03                                  */
/*                2.02  2005-06-12   minor change                   */
/*                2.03  2005-09-28   support linux2.6               */
/*                2.10  2005-12-27   support VSSP32                 */
/*                2.11  2006-02-02   add heart beat                 */
/*                2.12  2006-11-02   display board name             */
/*                2.13  2007-02-15   add one sigma value display    */
/*                2.14  2007-02-23   minor revision                 */
/*                2.20  2007-03-02   add log out when DC offset     */
/*                                   was set                        */
/*                2.21  2007-03-05   debug in DC offset set         */
/*                2.30  2007-05-13   add current sampler time       */
/*                                   display                        */
/*                                                                  */
/*      Copyright (c) 2002-2007 T.Kondo/NICT All Right Reserved     */
/*                                                                  */
/********************************************************************/
/*
  Monitoring K5/VSSP and K5/VSSP32 input level
         by T.Kondo 2002.5.26   
                    2002.10.03   remove 1pps sync 
                    2005.06.12   minor change 
                    2005.09.28   support linux2.6 
                    2005.12.27   support VSSP32 
                    2006.02.02   add heart beat 
                    2006.11.02   display which board is used
                    2007-02-15   add one sigma (standard deviation) of sgnals
                    2007-03-02   add log out when DC offset  was set
                    2007-03-05   debug in DC offset set
                    2007-05-13   add time display

   How to run
        monit numch [AUTO|ZERO]
            where  numch ---- number of channel
                   AUTO   --- Automatic DC offset set in case of K5/VSSP32
                              'A' or 'a' is also OK
                   ZERO   --- Reset DC offset in case of K5/VSSP32
                              'Z' or 'z' is OK

*/
#define PROGRAM_VERSION "Ver. 2007-05-13"

#include "chead.h"      // common header for IP-VLBI utilities
#include "libipvlbi_board.h"  // library for IP-VLBI sampler board

#define K5LOGDIR  "K5LOGDIR"  // environment default for log out directry
#define K5LOGDIRDFLT  "./"    // default log file directory

/* set global variables */
char dev[] = DEV_FILE;  // IP-VLBI BOARD control variable
int fd_in;                 // IP-VLBI BOARD control variable
logval_t lg;  // for log out

/* set interrupt handler */
void Handler(int n);

void SetHandler(void)
{
    if(signal(SIGINT, Handler) == SIG_ERR){
        printf("Signal handler set up failure!\n");
        abort();
    }
}

void Handler(int n)
{
     static int x=0;
     int rv;
     printf("Interrupt occurred !!\n");

     /* sampling stop */

     rv = ioctl(fd_in, TDSIO_SAMPLING_STOP);
     if (rv == -1) {
     perror("stop sampling failed, anyway stop ");
     goto end;
     }
     else {
        printf("Stop sampling!\n");
     }

     //close(fd_out);

 end:
     close(fd_in);
     exit(0);
}


void yts_to_k5time(int year, int totalday, int seconds, timeval_k5_t *k5time)
{
    /*  convert year, totalday, second to k5time variable */

    k5time->year=year; 
    k5time->ddd=totalday;
    k5time->second=seconds;
    ddd2md(year, totalday, &k5time->month, &k5time->mday);

    k5time->hh=seconds/3600;
    k5time->mm=(seconds/60)% 60; 
    k5time->ss=seconds % 60;
}

int check_head_sync(u_int8_t *pH, u_int8_t bHeadCode, timeval_k5_t *k5time, char *cbuf)
{
    int isecond,iyear,itotday;
    int ierr;
    ierr=-1;
    if ((*(pH) == 0xFF)&&(*(pH+1) == 0xFF)&&(*(pH+2) == 0xFF)&&(*(pH+3) == 0xFF)
           &&(*(pH+7)==bHeadCode)){   // sync found
          //printf ("Sync found: ");          // debug
          isecond = (*(int *)(pH + 4)) & 0x1FFFF;
          //printf ("Debug %02d:%02d:%02d \n",isecond/3600,(isecond/60)% 60,isecond % 60);    // Time
          if(bHeadCode==0x8C){  // VSSP32
             iyear=(((*(int *)(pH+8)) >> 9) & 0x3F)+2000;
             itotday=(*(int *)(pH+8)) & 0x1FF;
          } else {
             //k5time->year=0; k5time->month=0; k5time->mday=0;  k5time->ddd=0;
             iyear=k5time->year;
             itotday=k5time->ddd;
          }
          yts_to_k5time(iyear, itotday, isecond, k5time);
          sprintf(cbuf,"%04d/%02d/%02d (%03d) %02d:%02d:%02d\0",
                       k5time->year,k5time->month,k5time->mday,k5time->ddd,
                       k5time->hh,k5time->mm,k5time->ss);
          ierr=0;
     }
     return ierr;
}

void do_counts_for_statistics(int numch, u_int8_t *pData, int ndata, int nPosInDataFrame, 
                              int *nave, double *sum, double *sum2, int ihist[][8])
{
       int k,k1,w1,w2,w3;
       double d1;
       
       
       if( numch == 1 ) {
          w3 =0;
       } else {
          w3=nPosInDataFrame % 4;
          //switch(nPosInDataFrame % 4){
          //   case 0: k1=0; break;
          //   case 1: k1=3; break;
          //   case 2: k1=2; break;
          //   case 3: k1=1; 
          //}
       }
       
       for(k=0; k<ndata; k++){
           //for(k=0; k<ndata; k++){
            w1=(int)pData[k];
            w2=w1/32; 
            ihist[w3][w2]+=1;
            d1=(double)w1- 255.0/2.0;  // assuming 8 bit AD
            sum[w3]+=d1;  // for average calc
            sum2[w3]+=(d1*d1);  // for variance calc
            nave[w3]++;
            if(numch = 4 ){
               w3++;
               if( w3 == 4 ) w3=0;
            }
       }
}


void hist_plot(int numch, int *nave, double *sum, double *sum2, int ihist[][8],
               double *ave, double *sigma)
{
   /* Histgram Plot by normalized Level */
    int hmax[4];
    char iline[70],cbuf[70];
    char* lev0="          ";
    char* lev1="*         ";
    char* lev2="**        ";
    char* lev3="***       ";
    char* lev4="****      ";
    char* lev5="*****     ";
    char* lev6="******    ";
    char* lev7="*******   ";
    char* lev8="********  ";
    char* lev9="********* ";
    char* lev10="**********";
    char* levd="<         ";
    char* levu="         >";
    int k,mm,w1,w2,w3;
    //double ave[4],ave2[4],sigma[4];
    double ave2[4];
    
    /* Maximum found by Channel */
    for(mm=0; mm<numch; mm++){
       w3=0;
       for(k=0; k<8; k++){
          if(ihist[mm][k] > w3) w3=ihist[mm][k]; 
       }
       hmax[mm]=w3;
    }

    for(k=7; k>=0; k--){
       if(k==7 || k==0 ){
          if(k==7) strcpy(iline, "  + FULL ");
          if(k==0) strcpy(iline, "  - FULL ");
       } else {
          strcpy(iline, "         ");
       }
       for(mm=0; mm<numch; mm++){
          strcat(iline, "  |");
          w3=hmax[mm];
          if(w3 < 10) w3=10;
          w2=w3/10;
          w1=ihist[mm][k];
          //printf(" k = %d  ch = %d  W3 = %d  w1= %d\n",k,mm,w3,w1);
          w1=w1/w2;
          switch(w1){
             case  0: strcat(iline,lev0); break;
             case  1: strcat(iline,lev1); break;
             case  2: strcat(iline,lev2); break;
             case  3: strcat(iline,lev3); break;
             case  4: strcat(iline,lev4); break;
             case  5: strcat(iline,lev5); break;
             case  6: strcat(iline,lev6); break;
             case  7: strcat(iline,lev7); break;
             case  8: strcat(iline,lev8); break;
             case  9: strcat(iline,lev9); break;
             case 10: strcat(iline,lev10); break;
             default: strcat(iline,lev0);
          }
       }
       printf(" %s\n",iline);
       //printf(" k = %d  %d  %d %d %d\n",k,ihist[0][k],ihist[1][k],
        //                 ihist[2][k],ihist[3][k]);
        
       
   }
   
   
   // DC offset (average) and STD calc and plot
   for(k=0; k<numch; k++) {
       ave[k]=sum[k]/(double)nave[k];
       ave2[k]=sum2[k]/(double)nave[k];
       if(ave2[k]-ave[k]*ave[k] <= 0.0){
          sigma[k]=0.0;
       } else {
          sigma[k]=sqrt(ave2[k]-ave[k]*ave[k]);  // standard deviation
       }
   }

   /* print DC offset info */
   strcpy(iline, " DC OFFSET  ");
   for(mm=0; mm<numch; mm++){
      sprintf(cbuf,"%6.1f/256\0",ave[mm]);
      strcat(iline, cbuf);
      strcat(iline, "   ");
   }
   printf(" %s\n",iline);
   
   /* print standard deviation info */
   strcpy(iline, " ONE SIGMA  ");
   for(mm=0; mm<numch; mm++){
      sprintf(cbuf,"%6.1f/256\0",sigma[mm]);
      strcat(iline, cbuf);
      strcat(iline, "   ");
   }
   printf(" %s\n",iline);
}


/*  f[^W֐
    NۂɃf[^W܂łRbII(K5/VSSP̏ꍇj
    outfile ---- f[^o̓t@C
    span    ---- f[^Wԁ@ibj
    sfreq   ---- TvOg
                 40,100,200,500 (for kHz)
                   1,2,4,8,16 (for MHz)
    adbit   ---- `crbg
                   1,2,4,8
    numch   ---- `l
                    1, 4
*/
//int sampleget_monit(char* outfile, int span, int sfreq, 
//                    int adbit, int numch, int filter, int adjust)
int sampleget_monit(char* outfile, vssp_param_t *sp, int span, int adjust)
{
    //char outfile[] = "./tds.data";
    //char dev[] = "/dev/tds0";
    //int fd_in;
    int fd_out;
    mode_t ofilemode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
    int rv;
    unsigned int timen,day,sec; 
    unsigned int secold;
    int i,h,m,s;
    int secforce;
    int ihist[4][8];
    int w1,w2,w3,mm;
    int beat;
    double d1;
    //double sum[4],ave[4],sum2[4],ave2[4],sigma[4];
    double sum[4],ave[4],sum2[4],sigma[4];
    int nave[4];
    char cbuf[100], jikoku[40];
    char iline[70];
    int clineflag;
    int aveflg;  // average already calc flag
#ifdef _VSSP32
    struct tdsio tdsio;   // for setting DC_OFFSET or getting DC_OFFSET
#endif
    
    //int span, sfreq;
#ifdef _VSSP32
    int headbit=256;  // default header bit number  for K5/VSSP32
    int headbyte=32;  // header byte number
    char bHeadCode=0x8C;  // 2nd sync pattern
#else
    int headbit=64;  // header bit number  for K5/VSSP
    int headbyte=8;  // header byte number
    char bHeadCode=0x8B;  // 2nd sync pattern
#endif
    int allbyte;     // TvOoCg
    int bitsec;       // Pb̃rbg@iwb_[܂ށj
    int bytesec;       // Pb̃oCg@iwb_[܂ށj
    int byteread;  // bytes number to read at once
    int mloopmax;    // loop number to read 1 sec data
    int mloop;
    int c,k;
    int nkai;
    int err, adjusted;
    int dcoffset[4];
    //unsigned int sampling1, freq, width, stat;
    int sampling1, freq, width, stat, filter0;
    unsigned int ref, pps, rhz, pll;
    int ch, pos;
    u_int8_t sfreq_t, sbit_t, numch_t;
    u_int8_t sampling;
    //u_int32_t buf[BUFSIZ * 2];
    //u_int32_t buf[1024 * 2];   // obsoleted 2007-05-12
    u_int8_t buf[1024 * 4];  // 4K buffer
    //u_int8_t buf[1024*2];  // 4K buffer
    u_int8_t header[32];
    u_int8_t *pH;   // pointer to header start
    u_int8_t *pD;  // pointer to data start
    int nread,ndata,nPosInDataFrame, nHeader, nPosInHeader;
    int k1,k2,k3,k4;
    int hantei; // flag to identify header filled or not
    int isecond,iyear,itotday;
    timeval_k5_t k5time;
    unsigned int vtime;  // used only for case of VSSP
    float fsec,tofset;   // used only for case VSSP
    double telapsed;           // to measure processing time
    time_t tstart, tfinish;    //           "
    //printf("BUFSIZ   %d\n",BUFSIZ);   // BUFSIZ: 1024 for K5/VSSP   8192 for K5/VSSP32
    //int xdata[BUFSIZ * 16];  // f[^̃f[^[p
    //int xdata[1024 * 16];  // f[^̃f[^[p
    
    int nbyte = sizeof(buf);   // nbyte = 4096

    //printf(" span = %d\n",span);    // debug
    //printf(" sfreq= %d\n",sfreq);   // debug

    /* sampling duration check */
    if (span <= 0) {
       printf(" parameter span (duration in sec) should be positive!\n");
       // nothing to do
       return -1;
    }

    adjusted=0;

loopmain:

    /* sampler open */
    fd_in = open(dev, O_RDONLY);
    //fd_in = open("/dev/tds0", O_RDONLY);
    if (fd_in == -1) {
        perror("device open() error ");
        return -1;
    }
    

#ifdef _VSSP32
    if((rv=ioctl(fd_in,TDSIO_INIT)) == -1) {
        perror("ioctl() INIT failed ");
        goto end;
    }
#endif
    /* board status check */
    if((rv=check_vssp(fd_in, &stat, 1)) == -1) {
        perror("ioctl() status get failed ");
        goto end;
    }
    
    /* year date get in case of VSSP */
#ifndef _VSSP32
    rv = ioctl(fd_in, TDSIO_GET_TIME, &vtime);
    if (rv == -1) {
         perror("ioctl() time get failed ");
         close(fd_in);
         goto end;
     }
     k5time.ddd=TDS_GET_DAYS(vtime);
     // year get from PC
     tofset=0.0;
     getsystimef(&k5time.year, &k5time.month,&k5time.mday,
      &k5time.hh,&k5time.mm,&k5time.ss, &fsec, &tofset);
     
#endif

    /* sampling parameters set */
    if((rv=set_vssp_sparam(fd_in,sp,1))==-1){
        printf("sampling set failed \n");
        goto end;
    }

#ifdef _VSSP32
    /* DC offset reset */
    if(adjust==2){
        for(k=0; k<4; k++){
           tdsio.key=k;
           tdsio.value=0;
           if ((rv=ioctl(fd_in, TDSIO_SET_DCOFFSET, &tdsio)) == -1) {
              printf("ioctl() set DCoffset failed\n");
           }
        }
    }
 
    /* DC offset get */  
    //tdsio.key=3;   // debug
    //tdsio.value=-50;   // debug
    //if ((rv=ioctl(fd_in, TDSIO_SET_DCOFFSET, &tdsio)) == -1) {   // debug
    //    printf("ioctl() set DCoffset failed\n");   // debug
    //}   // debug
    for(k=0; k<4; k++){
       tdsio.key=k;
       if ((rv=ioctl(fd_in, TDSIO_GET_DCOFFSET, &tdsio)) == -1) {
          printf("ioctl() get DCoffset failed\n");
       }
       dcoffset[k]= tdsio.value;
       if(tdsio.value > 127) dcoffset[k]= tdsio.value-256;
    }
    
    
#endif
    
    if (sp->sfreq >= 40) {
       //bitsec=sp->sfreq*1000 + headbit ;
       bitsec=sp->sfreq*1000 ;
    } else { 
       //bitsec=sp->sfreq*1000*1000 + headbit ;
       bitsec=sp->sfreq*1000*1000 ;
    }

    bitsec=bitsec*sp->adbit*sp->numch;   // bit number per second only data bits

    bytesec=bitsec/8;
    allbyte=bytesec*span;
    if (allbyte > 2000000000){
       allbyte= 2000000000;
       secforce=allbyte/bytesec;
       printf(" # of bytes force to set to 2GB because it exceeds 2GB\n");  // debug
       printf(" Obserbasation span (sec) reduced to %d\n",secforce);  // debug
    }
    printf(" bitsec  = %d\n",bitsec);  // debug
    printf(" bytesec = %d\n",bytesec);  // debug
    printf(" allbyte = %d\n",allbyte);  // debug
    

    /* get sampling parameters from a board */
    //if ((rv=get_vssp_sparam(fd_in,&freq,&width,&ch,&filter0,1))== -1) {
    if ((rv=get_vssp_sparam(fd_in,sp,1))== -1) {
        perror("ioctl() get sampling parameter failed ");
        goto end;
    } else {
        printf("sampling setting is %d kHz %d BIT %d CH",
                  sp->freqkhz,sp->adbit,sp->numch);
#ifdef _VSSP32
        printf(" FILTER %d MHz\n",sp->filter);
#else
        printf("\n");
#endif
    }   

    rv = ioctl(fd_in, TDSIO_BUFFER_CLEAR);   // {[hFIFÕNA[
    if (rv == -1) {
        perror("ioctl() FIFO clear failed ");
        goto end;
    } else {
        printf("ioctl() FIFO clear succeed\n");
    }

    rv = ioctl(fd_in, TDSIO_SAMPLING_START);
    if (rv == -1) {
        perror("ioctl() start sampling failed ");
              printf("Probably time was not set for the IP-VLBI board.\n");
              printf("Please set time using timesettk and try again.\n");
        goto end;
    } else {
        printf("ioctl() start sampling succeed\n");
    }
    
    tstart= time(NULL);

  /* MONITOR START */
  
    printf("\033[2J");   //  Erase the screen ( move the cursor to home ) for VT100
    printf("\033[H");   //  move the cursor to home
    printf("********************************************************************\n");
    printf("*                                                                  *\n");
    printf("*        MONIT (signal level monitor for K5/VSSP or K5/VSSP32)     *\n");
    printf("*                by T.Kondo/NICT  (%s)                *\n",PROGRAM_VERSION);
#ifdef _VSSP32
    printf("*                    Target Board is K5/VSSP32                     *\n");
#else
    printf("*                     Target Board is K5/VSSP                      *\n");
#endif
    printf("*                                                                  *\n");
    printf("*                        CTRL C  for STOP                          *\n");
    printf("*                                                                  *\n");
    printf("*******T***********************************************KONDO********\n");
    printf("\n");
#ifdef _VSSP32
    printf("  DC_OFST_P    %5d        %5d        %5d       %5d\n",
                 dcoffset[0],dcoffset[1],dcoffset[2],dcoffset[3]);
#else
    printf("\n");
#endif
    printf("\n");

    if(sp->numch == 1) {
        printf("              CH 1\n");
    } else {
        printf("              CH 1         CH 2         CH 3         CH 4\n");
    }
    printf("\033[1A");  // move cursol 1 lines up
    c = 0;

    beat=0;
    nkai=0;
    
    nread=0;
    
    hantei=0;
    nPosInHeader=0;
    nHeader=0;
    // 1st header find
    rv = read(fd_in, buf, nbyte);
    if (rv == -1) {
          printf("read() failed   ");
          ioctl(fd_in, TDSIO_GET_ERROR, &err); printf("Error code = %d\n",err);  // debug for VSSP32
          tfinish= time(NULL);
          telapsed = difftime(tfinish,tstart);
          printf("Error occurred after %f minutes sampling.\n",telapsed/60.0);
          goto end;
    }
    pH = buf;
    while (nread < rv) {
         pH = (u_int8_t *)memchr(pH,bHeadCode,rv - nread);   // 2nd SYNC Pattern Search
         if (pH == NULL) {   // none SYNC Pattern ??
             nread = rv;  
         } else {                                               // Header Check
             //printf("2nd sync found! "); // debug
             pH -= 7;
             if ((*(pH) == 0xFF)&&(*(pH+1) == 0xFF)&&(*(pH+2) == 0xFF)&&(*(pH+3) == 0xFF)) {
                 nread = (int)(pH - buf);               // Header Point found
                 //printf ("Sync : %d\n",nread);          // 1st SYNC Header Point
                 isecond = (*(int *)(pH + 4)) & 0x1FFFF;
                 //printf ("%02ld:%02ld:%02ld \n",lTod/3600,(lTod/60)% 60,lTod % 60);    // Time
#ifdef _VSSP32
                    iyear=(((*(int *)(pH+8)) >> 9) & 0x3F)+2000;
                    itotday=(*(int *)(pH+8)) & 0x1FF;
#else
                    iyear=k5time.year;
                    itotday=k5time.ddd;
#endif
                  yts_to_k5time(iyear, itotday, isecond, &k5time);
                  sprintf(jikoku,"%04d/%02d/%02d (%03d) %02d:%02d:%02d\0",
                       k5time.year,k5time.month,k5time.mday,k5time.ddd,
                       k5time.hh,k5time.mm,k5time.ss);
                  //printf("%s\n",jikoku); // debug
                  nread = rv - nread;                      // after SYNC Header
                  //pD = pH +10;
                  pD = pH +headbyte;
                  ndata=nread - headbyte;
                  nPosInDataFrame=0;
                  mloop=0;
                  //nChkHeader = nHeader = 1;                       // 1st Header
                  nHeader = 1;                       // 1st Header
                  break;
              }
              pH += 8;                                       // NEXT Search point
              nread = (int)(pH - buf);                   // number of Search byte
          }
    }
    if(nHeader==0) {
        printf("TRY AGAIN\n");
        rv = ioctl(fd_in, TDSIO_SAMPLING_STOP);
        if (rv == -1) {
            perror("ioctl() stop sampling failed ");
            goto end;
        }
        else {
            printf("ioctl() stop sampling succeed\n");
        }
        close(fd_in);
        goto loopmain;
    }

    hantei=0;   // 0: header not updated
                // 1: header fully updated
                // 2: header partly updated
                
    /* histgram data initialize */
    for(w1=0; w1<4; w1++){
           for(k=0; k<8; k++){
              ihist[w1][k]=0;
           }
           sum[w1]=0.0;
           sum2[w1]=0.0;
           nave[w1]=0;
    }
    
    
    // first time label display
    printf("\033[3A");  // move cursol 3 lines up
    printf("                                           %s\n",jikoku);
    printf("\033[2B");  // move the cursor 2 line down

    clineflag=0; // cuurrent line number
    aveflg=0;    // avelage already calc flag
    
    while (1) {      // loop until CTRL-C
    
       if(ndata == 0){
           rv = read(fd_in, buf, nbyte);
           if (rv == -1) {
               printf("read() failed   ");
               ioctl(fd_in, TDSIO_GET_ERROR, &err); printf("Error code = %d\n",err);  // debug for VSSP32
               tfinish= time(NULL);
               telapsed = difftime(tfinish,tstart);
               printf("Error occurred after %f minutes sampling.\n",telapsed/60.0);
               goto end;
           }
           ndata=rv;
           pD=buf;
        }
        
        if(hantei==2){// rest of header copy to header array
            memcpy(header+nPosInHeader,buf,headbyte-nPosInHeader);
            pH=header;
            hantei=1;
            pD=buf+headbyte-nPosInHeader;
            ndata-=(headbyte-nPosInHeader);
            nPosInDataFrame=0;
        }
        
        if( nPosInDataFrame+ndata > bytesec) { //  header included
            //printf("Debug %d %d %d\n",nPosInDataFrame,ndata,bytesec);
            //do statistics
            if( nPosInDataFrame+ndata > bytesec+headbyte ){// full header can be retreaved
                 pD=buf; 
                 nread=bytesec-nPosInDataFrame;
                 do_counts_for_statistics(sp->numch, pD, nread, nPosInDataFrame, nave, sum, sum2, ihist);
                 //(bytesec-nPosInDataFrame)   <= supposed header start position
                 pH=buf+(bytesec-nPosInDataFrame);
                 nPosInHeader=headbyte;
                 hantei=1;
                 
                 pD=pH+headbyte;
                 nPosInDataFrame=0;
                 ndata-=(nread+headbyte);
                 //printf("Debug2 %d %d\n",ndata,headbyte);
                 //printf("Full Included %ld %ld %ld\n",buf,pH,pD);
            } else {   // header patrly included
                 pD=buf; 
                 ndata=bytesec-nPosInDataFrame;
                 
                 nPosInHeader=bytesec+headbyte-(nPosInDataFrame+ndata);
                 memcpy(header,buf+(bytesec-nPosInDataFrame),nPosInHeader);
                 hantei=2;
                 //printf("Partly Included %ld %ld %ld\n",buf,pH,pD);
            }
                 
        }

        if(hantei==1){ // check header and time info update
            if(check_head_sync(pH, bHeadCode, &k5time, jikoku)<0){ // sync detection error
                rv = ioctl(fd_in, TDSIO_SAMPLING_STOP);
                if (rv == -1) {
                    perror("ioctl() stop sampling failed ");
                    goto end;
                }
                else {
                    printf("ioctl() stop sampling succeed\n");
                }
                close(fd_in);
                goto loopmain;   // re-start
             }
             hantei=0;
       
             // update time display
            
             if(clineflag==0)
                  printf("\033[4A");  // move cursol 4 lines up
             else
                  printf("\033[14A");  // move cursol 14 lines up
             printf("                                           %s\n",jikoku);
             if(clineflag==0)
                  printf("\033[3B");  // move the cursor 3 line down
             else
                  printf("\033[13B");  // move cursol 13 lines down
 
        }

       /* Histgram Statistics */
       do_counts_for_statistics(sp->numch, pD, ndata, nPosInDataFrame, 
                                 nave, sum, sum2, ihist);

       nPosInDataFrame+=ndata;
       ndata=0;   // ndata reset for next loop

       if((c != 1) && (c !=0)) goto bypass1;  // to reduce the refresh rate
       
       if(c !=0 ) printf("\033[11A");  // move cursol 11 lines up
       

       /* heart beat */
       
       if(beat == 0) printf("  * \n");
       if(beat == 1) printf("   *\n");
       beat++;
       if(beat==2) beat=0;
       

       /* Histgram Plot by Level and display average and one sigma*/
       
       hist_plot(sp->numch, nave, sum, sum2,ihist, ave, sigma);
       
       aveflg=1;
       clineflag=1;
       
      /* histgram data initialize */
       
       for(w1=0; w1<4; w1++){
           for(k=0; k<8; k++){
              ihist[w1][k]=0;
           }
           sum[w1]=0.0;
           sum2[w1]=0.0;
           nave[w1]=0;
       }
       
       //printf("\n");

       bypass1:  ;
       
       c++;
       //printf("c=%d\n", c); printf("\033[10A"); // debug
       if(c == 40*sp->numch ) {
          c=1;
          nkai++;
          if(nkai == 3){
              nkai=0;
              if(adjust==1 && aveflg==1 ) break;  // exit while loop
          }
       }
    }


    rv = ioctl(fd_in, TDSIO_SAMPLING_STOP);
    if (rv == -1) {
        perror("ioctl() stop sampling failed ");
        goto end;
    }
    else {
        printf("ioctl() stop sampling succeed\n");
    }
    
    if(adjust==1){ // DC offset adjust
#ifdef _VSSP32
       printf(" Adjusting DC offset......\n");
       if((rv=ioctl(fd_in,TDSIO_INIT)) == -1) {
           perror("ioctl() INIT failed ");
           goto end;
       }
       vssp_wait_nsec(fd_in, 2);       // wait 2 seconds just for attraction
       for(mm=0; mm<sp->numch; mm++){
          /* DC offset set */  
          tdsio.key=mm;
          dcoffset[mm]= -(int)ave[mm];
          // get current DC offset
          if ((rv=ioctl(fd_in, TDSIO_GET_DCOFFSET, &tdsio)) == -1) {
             printf("ioctl() get DCoffset failed\n");
          }
          if(tdsio.value > 127) tdsio.value-=256;
          // set DC offset
          tdsio.value+=dcoffset[mm];
          printf("CH %d  DC_OFFSET adjusted value is %d\n",mm+1, tdsio.value);
          if ((rv=ioctl(fd_in, TDSIO_SET_DCOFFSET, &tdsio)) == -1) {
              printf("ioctl() set DCoffset failed\n");
          }
          vssp_wait_nsec(fd_in, 1);       // wait 2 seconds just for attraction
       }
       printf(" Adjusting Finished ......\n");
       //log out 
       sprintf(cbuf,"monit: VSSP32 OFFSET ADJUST CH1 = %d  CH2 = %d  CH3 = %d  CH4 = %d\0",
                 dcoffset[0],dcoffset[1],dcoffset[2],dcoffset[3]);
       LogWrite4(&lg, 0, 0.0, 0, cbuf);
       
#endif
       vssp_wait_nsec(fd_in, 2);       // wait 2 seconds just for attraction
       close(fd_in);
       adjust=0;
       aveflg=0;
       adjusted=1;
       goto loopmain;
    }

 end:
    close(fd_in);

    return rv;
}


int main(int argc, char **argv)
{
    char outfile[] = "./tds.data";
    char* dflt_logdir=K5LOGDIRDFLT;        // default log file directory
    char cbuf[20];
    int rv;
    int span, sfreq, adbit, numch, filter, adjust;
    vssp_param_t sp;

    if(argc <= 1 ){
        printf("Usage: monit numch [AUTO|ZERO]\n");
        printf("         where  numch ---- number of channel used\n");
        printf("                           1,4\n");
        printf("                AUTO --- automatic DC offset adjust (K5/VSSP32 only)\n");
        printf("                ZERO --- reset DC offset adjust (K5/VSSP32 only)\n");
        exit(0);
    }
    
    if (argc>=2){
         if(strncmp(*(argv+1),"env",3)==0){
           printf("   Enviroment variables\n");
           printf("        %s ---   default log file directory\n",K5LOGDIR);
           printf("                      ( %s )",getenv(K5LOGDIR));
           if(getenv(K5LOGDIR)==NULL) {
              printf(",  program deflt is ( %s )\n",K5LOGDIRDFLT);
           } else {
              printf("\n");
           }
           printf("\n");
           exit(0);
         } 
    } 

       
    sp.numch = atoi(*(argv + 1 ));
    if(sp.numch != 1) {sp.numch=4;}
    span=100;
    sp.sfreq=1;
    sp.adbit = 8;
    sp.filter=0;  // K5/VSSP32 parameter
    adjust=0;
    if (argc>=3) {
       strcpy(cbuf,*(argv+2));
       if(cbuf[0]=='A' || cbuf[0]=='a') adjust=1;
       if(cbuf[0]=='Z' || cbuf[0]=='z') adjust=2;
    }
    
    
    /* log directory name get from enviroment variable K5LOGDIR if it is set */
    if(getenv(K5LOGDIR) != NULL){
        strcpy(lg.logdir,getenv(K5LOGDIR));
    } else {
        strcpy(lg.logdir,dflt_logdir);
    }
    /* get host name */
    hostget(lg.hostname);
    
    lg.logfiletype=1;    //-1: no log   0: use given file name  1: create common log file name and append

/*
    printf(" span = %d\n",span);    // debug
    printf(" sfreq= %d\n",sfreq);   // debug
    printf(" adbit= %d\n",adbit);   // debug
    printf(" numch= %d\n",numch);   // debug
*/
    printf("*******************************************\n");
    printf("*  K5/VSSP (IP-VLBI) board signal monitor *\n");
    printf("*          MONIT (%s)        *\n",PROGRAM_VERSION);
    printf("*                by                       *\n");
    printf("*               NICT                      *\n");
    printf("*******************************************\n");
    printf(" \n");

    SetHandler();
    //rv=sampleget_monit(outfile, span, sfreq, adbit, numch, filter, adjust);
    rv=sampleget_monit(outfile, &sp, span, adjust);


    return 0;
}
