/*******************************************************************/
/*                                                                 */
/*   Program Name :  timesync                                      */
/*                                                                 */
/*      Version:  0.00  2002-07-01                                 */
/*                1.00  2002-10-04                                 */
/*                1.10  2003-06-17  add log out                    */
/*                1.11  2003-07-20  add env variable monitor       */
/*                1.12  2005-06-12  minor update                   */
/*                2.00  2005-12-29  support VSSP32                 */
/*                2.10  2007-02-23  bug fix for VSSP32             */
/*                                  revised to use common library  */
/*                                                                 */
/*      Copyright (c) 2002-2007 T.Kondo/NICT All Right Reserved    */
/*                                                                 */
/*******************************************************************/
/*

   Timesync syncs time of board to external 1PPS signal.

         by T.Kondo 2002.7.1

     How to run
            timesync [period]
                where   period  -- display period after time sync (sec)
                                   default is 10 sec

 2005-12-20  support VSSP32
 2006-02-23  bug fix for VSSP32 (1sec delay occurred) 

*/
#define TIMESYNC_VERSION "Ver. 2007-02-23"

#include "chead.h"      // common header for IP-VLBI utilities
#include "libipvlbi_board.h"  // library for IP-VLBI sampler board

#define K5LOGDIR  "K5LOGDIR"  // environment default for log out directry
#define K5LOGDIRDFLT  "./"    // default log file directory

int main(int argc, char **argv)
{
    int fd_in,rv;
    unsigned int time,day,sec; 
    unsigned int secold, secmax;
    int  yyyy, mm, dd; 
    int  h, m, s;
    int  y3; // 7 bits year  (0-127)
    int counter;
    int i;
    char* dflt_logdir=K5LOGDIRDFLT;
    char logdir[60]; 
    char hostname[20]; 
    char cbuf[80];
    
    if (argc>=2){
         if(strncmp(*(argv+1),"env",3)==0){
           printf("   Enviroment variables\n");
           printf("        %s ---   default log file directory\n",K5LOGDIR);
           printf("                      ( %s )",getenv(K5LOGDIR));
           if(getenv(K5LOGDIR)==NULL) {
              printf(",  program deflt is ( %s )\n",K5LOGDIRDFLT);
           } else {
              printf("\n");
           }
           printf("\n");
           exit(0);
         } 
    } 

    if(argc >= 2){
       secmax = atoi(*(argv + 1 ));
       if(secmax <= 0) secmax=10;
    } else {
          secmax=10;
    }

    /* log directory name get from enviroment variable K5LOGDIR if it is set */
    if(getenv(K5LOGDIR) != NULL){
        strcpy(logdir,getenv(K5LOGDIR));
    } else {
        strcpy(logdir,dflt_logdir);
    }

    /* get host name */
    hostget(hostname);

    sprintf(cbuf,"timesync %s",TIMESYNC_VERSION);
    if(LogWrite2(logdir,hostname,0,0.0,0,cbuf) == -1) printf("LogWrite2 error!\n");

    fd_in = open(DEV_FILE, O_RDONLY); 
    if (fd_in == -1) {
        perror("device open() error ");
        return -1;
    }
#ifdef _VSSP32
    printf("K5/VSSP32 (IP-VLBI) Board 1PPS Time Sync %s\n",TIMESYNC_VERSION);
    // time get from device
    time=0;
    secold=90000;
    counter=0;
    while(1){
       rv = ioctl(fd_in, TDSIO_GET_YEAR, &y3);
       yyyy=y3+2000;
       rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
       if (rv == -1) {
          perror("ioctl() time get failed ");
          return -1;
       }
       day=TDS_GET_DAYS(time);
       sec=TDS_GET_SEC(time);
       sec+=1;    // for sync delay adjust
       if (sec > 86400){
          sec-=86400;
          day+=1;
       }
       if (counter > 0 && sec!=secold) {
          sec2hms(sec,&h,&m,&s);
          time = TDS_MAKE_TIME(day,sec);
          // year set
          rv = ioctl(fd_in, TDSIO_SET_YEAR, &y3);  
          if (rv == -1) {
              perror("ioctl() year set failed ");
              sprintf(cbuf,"ioctl() year set failed");
              if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
              return -1;
          }
          //date and seconds set to sampler
          rv = ioctl(fd_in, TDSIO_SET_TIME, &time);  
          if (rv == -1) {
              perror("ioctl() time set failed ");
              sprintf(cbuf,"ioctl() time set failed");
              if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
              return -1;
          }
          break;
       }
       secold=sec;
       counter++;
    }
    
    
#else
    printf("K5/VSSP (IP-VLBI) Board 1PPS Time Sync %s\n",TIMESYNC_VERSION);
#endif
    //sync to 1PPS
    rv = ioctl(fd_in, TDSIO_SYNC_1PPS);
    if (rv == -1) {
        perror("ioctl() 1pps sync failed ");
        return -1;
    }
    printf("1pps sync ok\n");

    time = 0; 
    secold=90000;
    counter=0;

    close(fd_in);
    // log out
    sprintf(cbuf,"timesync (OK)");
    if(LogWrite2(logdir,hostname,1,0.0,1,cbuf) == -1) printf("LogWrite2 error!\n");
    

    //get time from device and display while
    printf("\n");
    i=board_time_disp(secmax);

    printf("\n\n");
    return 0;
}
